@@ -1,4 +1,4 @@
-/*
+ /*
  * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
@@ -41,7 +41,10 @@ enum {
 	kKeyLargoVersion2			= 2,
 
     kPangeaDeviceId25			= 0x25,				// Pangea
-	kPangeaVersion0				= 0
+    kPangeaVersion0				= 0,
+        
+    kIntrepidDeviceId3e			= 0x3e,				// Intrepid
+    kIntrepidVersion0			= 0    
 };
 
 enum {
@@ -50,6 +53,7 @@ enum {
 	kKeyLargoFCRBase						= 0x00038,
 	kKeyLargoFCRCount						= 5,
 	kPangeaFCRCount							= 6,
+	kIntrepidFCRCount						= 6,
 	kKeyLargoFCR0							= 0x00038,
 	kKeyLargoFCR1							= 0x0003C,
 	kKeyLargoFCR2							= 0x00040,
@@ -75,7 +79,7 @@ enum {
 	kKeyLargoMB1Enable						= 1 << 24,			// MB1_Enable_h
   
 	// Feature Control Register 0 Definitions
-	kKeyLargoFCR0ChooseSCCB					= 1 << 0,			// ChooseSCCB (KeyLargo only - reserved on Pangea)
+	kKeyLargoFCR0ChooseSCCB					= 1 << 0,			// ChooseSCCB (KeyLargo only - reserved on Pangea and Intrepid)
 	kKeyLargoFCR0ChooseSCCA					= 1 << 1,			// ChooseSCCA
 	kKeyLargoFCR0SlowSccPClk				= 1 << 2,			// SlowSccPClk_h
 	kKeyLargoFCR0ResetSCC					= 1 << 3,			// ResetScc_h
@@ -83,43 +87,47 @@ enum {
 	kKeyLargoFCR0SccBEnable					= 1 << 5,			// SccBEnable_h
 	kKeyLargoFCR0SccCellEnable				= 1 << 6,			// SCC_Cell_EN_h
 	kKeyLargoFCR0ChooseVIA					= 1 << 7,			// ChooseVIA
-	
-	kKeyLargoFCR0HighBandFor1MB				= 1 << 8,			// HighBandFor1MB (KeyLargo only - reserved on Pangea)
-	kKeyLargoFCR0UseIRSource2				= 1 << 9,			// UseIRSource2 (KeyLargo only - reserved on Pangea)
+	kKeyLargoFCR0HighBandFor1MB				= 1 << 8,			// HighBandFor1MB (KeyLargo only - reserved on Pangea and Intrepid)
+	kKeyLargoFCR0UseIRSource2				= 1 << 9,			// UseIRSource2 (KeyLargo only - reserved on Pangea and Intrepid)
 	kKeyLargoFCR0UseIRSource1				= 1 << 10,			// UseIRSource1 (KeyLargo only)
-	kPangeaFCR0USB0PMIEnable				= 1 << 10,			// USB0_PMI_En_h (Pangea only)
+	kPangeaFCR0USB0PMIEnable				= 1 << 10,			// USB0_PMI_En_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDASWReset				= 1 << 11,			// IRDA_SW_Reset_h (KeyLargo only)
-	kPangeaFCR0USB0RefSuspendSel			= 1 << 11,			// USB0_RefSuspendSel_h (Pangea only)
+	kPangeaFCR0USB0RefSuspendSel			= 1 << 11,			// USB0_RefSuspendSel_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDADefault1				= 1 << 12,			// IRDA_Default1 (KeyLargo only)
-	kPangeaFCR0USB0RefSuspend				= 1 << 12,			// USB0_RefSuspend_h (Pangea only)
+	kPangeaFCR0USB0RefSuspend				= 1 << 12,			// USB0_RefSuspend_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDADefault0				= 1 << 13,			// IRDA_Default0 (KeyLargo only)
-	kPangeaFCR0USB0PadSuspendSel			= 1 << 13,			// USB0_PadSuspendSel_h (Pangea only)
+	kPangeaFCR0USB0PadSuspendSel			= 1 << 13,			// USB0_PadSuspendSel_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDAFastCon				= 1 << 14,			// IRDA_FAST_CON (KeyLargo only)
-	kPangeaFCR0USB1PMIEnable				= 1 << 14,			// USB0_PadSuspendSel_h (Pangea only)
+	kPangeaFCR0USB1PMIEnable				= 1 << 14,			// USB0_PadSuspendSel_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDAEnable					= 1 << 15,			// IRDAEnable_h (KeyLargo only)
-	kPangeaFCR0USB1RefSuspendSel			= 1 << 15,			// USB1_RefSuspendSel_h (Pangea only)
-	
+	kPangeaFCR0USB1RefSuspendSel			= 1 << 15,			// USB1_RefSuspendSel_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDAClk32Enable			= 1 << 16,			// IRDA_Clk32_EN_h (KeyLargo only)
-	kPangeaFCR0USB1RefSuspend				= 1 << 16,			// USB1_RefSuspend_h (Pangea only)
+	kPangeaFCR0USB1RefSuspend				= 1 << 16,			// USB1_RefSuspend_h (Pangea and Intrepid only)
 	kKeyLargoFCR0IRDAClk19Enable			= 1 << 17,			// IRDA_Clk19_EN_h (KeyLargo only)
-	kPangeaFCR0USB1PadSuspendSel			= 1 << 17,			// USB1_PadSuspendSel_h (Pangea only)
+	kPangeaFCR0USB1PadSuspendSel			= 1 << 17,			// USB1_PadSuspendSel_h (Pangea and Intrepid only)
 	kKeyLargoFCR0USB0PadSuspend0			= 1 << 18,			// USB0_PadSuspend0_h
 	kKeyLargoFCR0USB0PadSuspend1			= 1 << 19,			// USB0_PadSuspend1_h
 	kKeyLargoFCR0USB0CellEnable				= 1 << 20,			// USB0_Cell_EN_h
 	kKeyLargoFCR0USB1PadSuspend0			= 1 << 22,			// USB1_PadSuspend0_h
 	kKeyLargoFCR0USB1PadSuspend1			= 1 << 23,			// USB1_PadSuspend1_h
-	
 	kKeyLargoFCR0USB1CellEnable				= 1 << 24,			// USB1_Cell_EN_h
-	kKeyLargoFCR0USBRefSuspend				= 1 << 28,			// USB_RefSuspend_h
+	kKeyLargoFCR0USBRefSuspend				= 1 << 28,			// USB_RefSuspend_h (KeyLargo only - reserved on Pangea and Intrepid)
   
 	// Feature Control Register 1 Definitions
-	kKeyLargoFCR1AudioSel22MClk				= 1 << 1,			// Audio_Sel22MClk
-	kKeyLargoFCR1AudioClkEnable				= 1 << 3,			// AudClkEnBit_h
-	kKeyLargoFCR1AudioClkOutEnable			= 1 << 5,			// AudioClkOut_EN_h
-	kKeyLargoFCR1AudioCellEnable			= 1 << 6,			// AUDIO_Cell_EN_h
-	kKeyLargoFCR1ChooseAudio				= 1 << 7,			// ChooseAudio
+	kIntrepidFCR1USB2PMIEnable				= 1 << 0,			// USB2_PMI_En_h (Intrepid only)
+	kKeyLargoFCR1AudioSel22MClk				= 1 << 1,			// Audio_Sel22MClk (KeyLargo and Pangea)
+	kIntrepidFCR1USB2RefSuspendSel			= 1 << 1,			// USB2_RefSuspendSel_h (Intrepid only)
+	kIntrepidFCR1USB2RefSuspend				= 1 << 2,			// USB2_RefSuspend_h (Intrepid only)
+	kKeyLargoFCR1AudioClkEnable				= 1 << 3,			// AudClkEnBit_h (KeyLargo and Pangea)
+	kIntrepidFCR1USB2PadSuspendSel			= 1 << 3,			// USB2_PadSuspendSel_h (Intrepid only)
+	kIntrepidFCR1USB2PadSuspend0			= 1 << 4,			// USB2_PadSuspend0_h (Intrepid only)
+	kKeyLargoFCR1AudioClkOutEnable			= 1 << 5,			// AudioClkOut_EN_h (KeyLargo and Pangea)
+	kIntrepidFCR1USB2PadSuspend1			= 1 << 5,			// USB2_PadSuspend1_h (Intrepid only)
+	kKeyLargoFCR1AudioCellEnable			= 1 << 6,			// AUDIO_Cell_EN_h (KeyLargo and Pangea)
+	kIntrepidFCR1USB2CellEnable				= 1 << 6,			// USB2_Cell_EN_h (Intrepid only)
+	kKeyLargoFCR1ChooseAudio				= 1 << 7,			// ChooseAudio (KeyLargo and Pangea)
 	
-	kKeyLargoFCR1ChooseI2S0					= 1 << 9,			// ChooseI2S0 (KeyLargo only - reserved on Pangea)
+	kKeyLargoFCR1ChooseI2S0					= 1 << 9,			// ChooseI2S0 (KeyLargo only - reserved on Pangea and Intrepid)
 	kKeyLargoFCR1I2S0CellEnable				= 1 << 10,			// I2S0_Cell_EN_h
 	kKeyLargoFCR1I2S0ClkEnable				= 1 << 12,			// I2S0_ClkEnBit_h
 	kKeyLargoFCR1I2S0Enable					= 1 << 13,			// I2S0Enable_h
@@ -127,43 +135,67 @@ enum {
 	kKeyLargoFCR1I2S1CellEnable				= 1 << 17,			// I2S1_Cell_EN_h
 	kKeyLargoFCR1I2S1ClkEnable				= 1 << 19,			// I2S1_ClkEnBit_h
 	kKeyLargoFCR1I2S1Enable					= 1 << 20,			// I2S1Enable_h
-	kKeyLargoFCR1EIDE0Enable				= 1 << 23,			// EIDE0_EN_h (KeyLargo only - reserved on Pangea)
+	kKeyLargoFCR1EIDE0Enable				= 1 << 23,			// EIDE0_EN_h (KeyLargo and Intrepid - reserved on Pangea)
 	
-	kKeyLargoFCR1EIDE0Reset					= 1 << 24,			// EIDE0_Reset_l (KeyLargo only - reserved on Pangea)
-	kKeyLargoFCR1EIDE1Enable				= 1 << 26,			// EIDE1_EN_h (KeyLargo only - reserved on Pangea)
-	kKeyLargoFCR1EIDE1Reset					= 1 << 27,			// EIDE1_Reset_l (KeyLargo only - reserved on Pangea)
-	kKeyLargoFCR1UIDEEnable					= 1 << 29,			// IDE_UD_EN_h
-	kKeyLargoFCR1UIDEReset					= 1 << 30,			// UltraIDE_Reset_l
+	kKeyLargoFCR1EIDE0Reset					= 1 << 24,			// EIDE0_Reset_l (KeyLargo and Intrepid only - reserved on Pangea)
+	kKeyLargoFCR1EIDE1Enable				= 1 << 26,			// EIDE1_EN_h (KeyLargo only - reserved on Pangea and Intrepid)
+	kKeyLargoFCR1EIDE1Reset					= 1 << 27,			// EIDE1_Reset_l (KeyLargo only - reserved on Pangea and Intrepid)
+	kKeyLargoFCR1UIDEEnable					= 1 << 29,			// IDE_UD_EN_h (KeyLargo and Pangea only - reserved on Intrepid)
+	kKeyLargoFCR1UIDEReset					= 1 << 30,			// UltraIDE_Reset_l (KeyLargo and Pangea only - reserved on Intrepid)
+  
   
 	// Feature Control Register 2 Definitions
 	kKeyLargoFCR2IOBusEnable				= 1 << 1,			// IOBus_EN_h
 	kKeyLargoFCR2SleepState					= 1 << 8,			// SleepStateBit_h (KeyLargo only)
 	kPangeaFCR2StopAllKLClocks				= 1 << 8,			// StopAllKLClocks (Pangea only)
 	kKeyLargoFCR2MPICEnable					= 1 << 17,			// MPIC_Enable_h
-	kPangeaCardSlotReset					= 1 << 18,			// CardSlot_Reset_h (Pangea only - reserved on KeyLargo)
+	kPangeaCardSlotReset					= 1 << 18,			// CardSlot_Reset_h (Pangea and Intrepid only - reserved on KeyLargo)
 	kKeyLargoFCR2AltDataOut					= 1 << 25,			// AltDataOut - spare on Pangea
   
 	// Feature Control Register 3 Definitions
-	kKeyLargoFCR3ShutdownPLLTotal			= 1 << 0,			// Shutdown_PLL_Total
-	kKeyLargoFCR3ShutdownPLLKW6				= 1 << 1,			// Shutdown_PLLKW6
-	kKeyLargoFCR3ShutdownPLLKW4				= 1 << 2,			// Shutdown_PLLKW4
-	kKeyLargoFCR3ShutdownPLLKW35			= 1 << 3,			// Shutdown_PLLKW35
+	kKeyLargoFCR3ShutdownPLLTotal			= 1 << 0,			// Shutdown_PLL_Total (Pangea and KeyLargo only - reserved on Intrepid)
+	kKeyLargoFCR3ShutdownPLLKW6				= 1 << 1,			// Shutdown_PLLKW6 (Pangea and KeyLargo)
+	kIntrepidFCR3ShutdownPLL3				= 1 << 1,			// Shutdown_PLL3 (Intrepid only)
+	kKeyLargoFCR3ShutdownPLLKW4				= 1 << 2,			// Shutdown_PLLKW4 (Pangea and KeyLargo)
+	kIntrepidFCR3ShutdownPLL2				= 1 << 2,			// Shutdown_PLL2 (Intrepid only)
+	kKeyLargoFCR3ShutdownPLLKW35			= 1 << 3,			// Shutdown_PLLKW35 (Pangea and KeyLargo)
+	kIntrepidFCR3ShutdownPLL1				= 1 << 3,			// Shutdown_PLL1 (Intrepid only)
 	kKeyLargoFCR3ShutdownPLLKW12			= 1 << 4,			// Shutdown_PLLKW12 (KeyLargo only - reserved on Pangea)
-	kKeyLargoFCR3PLLReset					= 1 << 5,			// PLL_Reset
-	kKeyLargoFCR3ShutdownPLL2X				= 1 << 7,			// Shutdown_PLL2X (KeyLargo only - reserved on Pangea)
+	kIntrepidFCR3EnablePll3Shutdown			= 1 << 4,			// EnablePLL3Shutdown (Intrepid only)
+	kKeyLargoFCR3PLLReset					= 1 << 5,			// PLL_Reset (Pangea and KeyLargo)
+	kIntrepidFCR3EnablePLL2Shutdown			= 1 << 5,			// EnablePLL2Shutdown (Intrepid only)
+	kIntrepidFCR3EnablePLL1Shutdown			= 1 << 6,			// EnablePLL1Shutdown (Intrepid only)
+	kKeyLargoFCR3ShutdownPLL2X				= 1 << 7,			// Shutdown_PLL2X (KeyLargo only - reserved on Pangea and Intrepid)
 	
-	kKeyLargoFCR3Clk66Enable				= 1 << 8,			// Clk66_EN_h (KeyLargo only - reserved on Pangea)
+	kKeyLargoFCR3Clk66Enable				= 1 << 8,			// Clk66_EN_h (KeyLargo only - reserved on Pangea and Intrepid)
 	kKeyLargoFCR3Clk49Enable				= 1 << 9,			// Clk49_EN_h
 	kKeyLargoFCR3Clk45Enable				= 1 << 10,			// Clk45_EN_h
-	kKeyLargoFCR3Clk31Enable				= 1 << 11,			// Clk31_EN_h
+	kKeyLargoFCR3Clk31Enable				= 1 << 11,			// Clk31_EN_h (KeyLargo and Pangea only - reserved on Intrepid)
 	kKeyLargoFCR3TimerClk18Enable			= 1 << 12,			// TIMER_Clk18_EN_h
 	kKeyLargoFCR3I2S1Clk18Enable			= 1 << 13,			// I2S1_Clk18_EN_h
 	kKeyLargoFCR3I2S0Clk18Enable			= 1 << 14,			// I2S0_Clk18_EN_h
-	kKeyLargoFCR3ViaClk16Enable				= 1 << 15,			// VIA_Clk16_EN_h
+	kKeyLargoFCR3ViaClk16Enable				= 1 << 15,			// VIA_Clk16_EN_h (KeyLargo & Pangea only)
+	kIntrepidFCR3ViaClk32Enable				= 1 << 15,			// VIA_Clk32_EN_h (Intrepid only)
 	
 	kKeyLargoFCR3Stopping33Enabled			= 1 << 19,			// Stopping33Enabled_h (KeyLargo only)
 	kPangeaFCR3PLLEnableTest				= 1 << 19,			// PLL_EN_TST (Pangea only)
   
+	kIntrepidFCR3Port5DisconnectSelect		= 1 << 16,			// Port5_DisconnectSelect (Intrepid only)
+	kIntrepidFCR3Port5ConnectSelect			= 1 << 17,			// Port5_ConnectSelect (Intrepid only)
+	kIntrepidFCR3Port5ResumeSelect			= 1 << 18,			// Port5_ResumeSelect (Intrepid only)
+	kIntrepidFCR3Port5Enable				= 1 << 19, 			// Port5_Enable (Intrepid only)
+	kIntrepidFCR3Port5Disconnect			= 1 << 20, 			// Port5_Disconnect (Intrepid only)
+	kIntrepidFCR3Port5Connect				= 1 << 21, 			// Port5_Connect (Intrepid only)
+	kIntrepidFCR3Port5Resume				= 1 << 22, 			// Port5_Resume (Intrepid only)
+	
+	kIntrepidFCR3Port6DisconnectSelect		= 1 << 24,			// Port6_DisconnectSelect (Intrepid only)
+	kIntrepidFCR3Port6ConnectSelect			= 1 << 25,			// Port6_ConnectSelect (Intrepid only)
+	kIntrepidFCR3Port6ResumeSelect			= 1 << 26,			// Port6_ResumeSelect (Intrepid only)
+	kIntrepidFCR3Port6Enable				= 1 << 27, 			// Port6_Enable (Intrepid only)
+	kIntrepidFCR3Port6Disconnect			= 1 << 28, 			// Port6_Disconnect (Intrepid only)
+	kIntrepidFCR3Port6Connect				= 1 << 29, 			// Port6_Connect (Intrepid only)
+	kIntrepidFCR3Port6Resume				= 1 << 30, 			// Port6_Resume (Intrepid only)
+
 	// Feature Control Register 4 Definitions
 	kKeyLargoFCR4Port1DisconnectSelect		= 1 << 0,			// Port1_DisconnectSelect
 	kKeyLargoFCR4Port1ConnectSelect			= 1 << 1,			// Port1_ConnectSelect
@@ -197,9 +229,9 @@ enum {
 	kKeyLargoFCR4Port4Connect				= 1 << 29,			// Port4_Connect
 	kKeyLargoFCR4Port4Resume				= 1 << 30,			// Port4_Resume
 	
-	// Feature Control Register 5 Definitions (Pangea only, FCR5 does not exist on KeyLargo)
-	kPangeaFCR5ViaUseClk31					= 1 << 0,				// ViaUseClk31
-	kPangeaFCR5SCCUseClk31					= 1 << 1,				// SCCUseClk31
+	// Feature Control Register 5 Definitions (Pangea and Intrepid only, FCR5 does not exist on KeyLargo)
+	kPangeaFCR5ViaUseClk31					= 1 << 0,				// ViaUseClk31 (Pangea only, reserved on Intrepid)
+	kPangeaFCR5SCCUseClk31					= 1 << 1,				// SCCUseClk31 (Pangea only, reserved on Intrepid)
 	kPangeaFCR5PwmClk32Enable				= 1 << 2,				// PwmClk32_EN_h
 	kPangeaFCR5Clk3_68Enable				= 1 << 4,				// Clk3_68_EN_h
 	kPangeaFCR5Clk32Enable					= 1 << 5,				// Clk32_EN_h
@@ -354,10 +386,64 @@ enum {
                                                 
     // Feature Control Register 4 Sleep Settings    
     kPangeaFCR4SleepBitsSet			=	0,
-	
     kPangeaFCR4SleepBitsClear		=	0
 };
 
+// desired state of Intrepid FCR registers when sleeping
+enum {
+	// Feature Control Register 0 Sleep Settings for Intrepid
+    kIntrepidFCR0SleepBitsSet		=	0,
+	
+    kIntrepidFCR0SleepBitsClear		=	kKeyLargoFCR0USB1CellEnable |
+										kKeyLargoFCR0USB0CellEnable |
+										kKeyLargoFCR0SccCellEnable |
+										kKeyLargoFCR0SccBEnable |
+										kKeyLargoFCR0SccAEnable,
+
+    // Feature Control Register 1 Sleep Settings
+    kIntrepidFCR1SleepBitsSet		=	0,
+	
+    kIntrepidFCR1SleepBitsClear		=	kIntrepidFCR1USB2CellEnable |
+										kKeyLargoFCR1I2S1Enable |
+										kKeyLargoFCR1I2S1ClkEnable |
+										kKeyLargoFCR1I2S1CellEnable |
+										kKeyLargoFCR1I2S0Enable |
+										kKeyLargoFCR1I2S0ClkEnable |
+										kKeyLargoFCR1I2S0CellEnable,
+                                                
+    // Feature Control Register 2 Sleep Settings
+    kIntrepidFCR2SleepBitsSet		=	kKeyLargoFCR2AltDataOut,
+	
+    kIntrepidFCR2SleepBitsClear		=	0,
+
+    // Feature Control Register 3 Sleep and Restart Settings
+    kIntrepidFCR3SleepBitsSet		=	kIntrepidFCR3ShutdownPLL3 |
+										kIntrepidFCR3ShutdownPLL2 |
+										kIntrepidFCR3ShutdownPLL1,
+										
+    kIntrepidFCR3SleepBitsClear		=	kKeyLargoFCR3ViaClk16Enable |
+										kKeyLargoFCR3I2S0Clk18Enable |
+										kKeyLargoFCR3I2S1Clk18Enable |
+										kKeyLargoFCR3TimerClk18Enable |
+										kKeyLargoFCR3Clk45Enable |
+										kKeyLargoFCR3Clk49Enable,	
+
+    // do not turn off SPI interface when restarting
+	
+    kIntrepidFCR3RestartBitsSet		=	kIntrepidFCR3ShutdownPLL3 |
+										kIntrepidFCR3ShutdownPLL2 |
+										kIntrepidFCR3ShutdownPLL1,
+										
+    kIntrepidFCR3RestartBitsClear	=	kKeyLargoFCR3I2S0Clk18Enable |
+										kKeyLargoFCR3I2S1Clk18Enable |
+										kKeyLargoFCR3Clk45Enable |
+										kKeyLargoFCR3Clk49Enable,
+                                                
+    // Feature Control Register 4 Sleep Settings    
+    kIntrepidFCR4SleepBitsSet		=	0,
+	kIntrepidFCR4SleepBitsClear		=	0
+};
+
 	// Offsets for the registers we wish to save.
 	// These come (almost) unchanged from the MacOS9 Power
 	// Manager plug-in (p99powerplugin.h)
@@ -469,6 +555,10 @@ private:
  
 	// Remember if the media bay needs to be turnedOn:
 	bool		   	mediaIsOn;
+	
+	// remember if we need to keep the SCC enabled during sleep
+	bool			keepSCCenabledInSleep;
+	
 	void			EnableSCC(bool state, UInt8 device, bool type);
 	void			PowerModem(bool state);
 	void 			ModemResetLow();
@@ -645,6 +735,7 @@ public:
 
 	virtual void      turnOffKeyLargoIO(bool restart);
 	virtual void      turnOffPangeaIO(bool restart);
+	virtual void      turnOffIntrepidIO(bool restart);
 
 	virtual void      powerWireless(bool powerOn);
 
